/// <reference types="react" />
import * as React from "react";
import { FormState } from "./Form";
import { StateEngine } from "./stateEngine";
import { FieldResult } from "./validation/index";
import { FieldValue } from "./Form";
import { SyntheticEvent } from "react";
/**
 * These props are provided to the component provided as renderer
 * to the Field component.  T is the type of the render components
 * own props, that will be provided via Fields renderProps prop
 */
export interface InjectedFieldProps<T extends object | void = {}> {
    meta: {
        valid: boolean;
        error?: string;
        pristine: boolean;
        touched: boolean;
        active: boolean;
        isValidating: boolean;
    };
    input: {
        onChange: FieldEventHandler;
        onFocus: FieldEventHandler;
        onBlur: FieldEventHandler;
        value: FieldValue | undefined;
        name: string;
    };
    ownProps: T;
}
/**
 * Field meta values provided by form-and-function
 */
export interface FieldMeta {
    touched: boolean;
    active: boolean;
    validation: FieldResult;
    isValidating: boolean;
}
export interface SetValue {
    value?: FieldValue;
}
export declare type FieldEventHandler = (e: SyntheticEvent<HTMLInputElement>, setValue?: SetValue) => void;
/**
 * Props passed to the injected Field component when it is used
 */
export interface FieldProps<T extends object | void = void> {
    name: string;
    renderProps?: T;
    render: React.SFC<InjectedFieldProps<T>>;
    onChange?: FieldEventHandler;
    onFocus?: FieldEventHandler;
    onBlur?: FieldEventHandler;
}
export interface FieldRecordAny<T = FieldMeta | Partial<FieldMeta>> {
    value: FieldValue | undefined;
    meta: T;
}
export declare type FieldRecord = FieldRecordAny<FieldMeta>;
export declare type FieldRecordUpdate = FieldRecordAny<Partial<FieldMeta>>;
/**
 * This is provided by the Form component when it calls makeField to
 * allow the Field to retrieve state from Form, and to indicate when
 * its value has changed
 */
export interface FormActions {
    onChange: (name: string, newValue: FieldValue | undefined) => void;
    getInitialValue: (name: string) => FieldValue | undefined;
}
/**
 * formActions allow the field to trigger functions within the parent
 * stateEngine allows the field to update and retrieve values from its parents state
 */
export declare const makeField: (formActions: FormActions, stateEngine: StateEngine<FormState>, resetState?: boolean) => {
    new <T extends object>(props: FieldProps<T>): {
        updateState: (fieldState: Partial<FieldRecordAny<Partial<FieldMeta>>>) => Promise<void>;
        handleFocus: (e: React.SyntheticEvent<any>) => void;
        handleBlur: (e: React.SyntheticEvent<any>) => void;
        handleChange: (e: React.SyntheticEvent<any>, setValue?: SetValue | undefined) => void;
        render(): React.ReactElement<any> | null;
        setState<K extends never>(state: void | ((prevState: void, props: FieldProps<T>) => void | Pick<void, K>) | Pick<void, K>, callback?: (() => void) | undefined): void;
        forceUpdate(callBack?: (() => void) | undefined): void;
        props: Readonly<{
            children?: React.ReactNode;
        }> & Readonly<FieldProps<T>>;
        state: void;
        context: any;
        refs: {
            [key: string]: React.ReactInstance;
        };
    };
    defaultProps: Partial<FieldProps<any>>;
};
